@@ -55,6 +55,13 @@ void grd_session_vnc_move_cursor (GrdSessionVnc *session_vnc,
                                   int            x,
                                   int            y);
 
-int grd_session_vnc_get_framebuffer_stride (GrdSessionVnc *session_vnc);
+void grd_session_vnc_set_client_clipboard_text (GrdSessionVnc *session_vnc,
+                                                char          *text,
+                                                int            text_length);
+
+int grd_session_vnc_get_stride_for_width (GrdSessionVnc *session_vnc,
+                                          int            width);
+
+gboolean grd_session_vnc_is_client_gone (GrdSessionVnc *session_vnc);
 
 #endif /* GRD_SESSION_VNC_H */
